@@ -10,6 +10,7 @@ from typing import List
 from mitosheet.code_chunks.code_chunk import CodeChunk
 from mitosheet.transpiler.transpile_utils import (
     column_header_list_to_transpiled_code, column_header_to_transpiled_code)
+from mitosheet.types import ColumnID, ColumnHeader
 
 LOOKUP = 'lookup'
 UNIQUE_IN_LEFT = 'unique in left'
@@ -30,40 +31,42 @@ class MergeCodeChunk(CodeChunk):
         return f'Merged {df_one_name} and {df_two_name} into {df_new_name}'
 
     def get_code(self) -> List[str]:
-        how = self.get_param('how') 
-        sheet_index_one = self.get_param('sheet_index_one') 
-        merge_key_column_id_one = self.get_param('merge_key_column_id_one') 
-        selected_column_ids_one = self.get_param('selected_column_ids_one') 
-        sheet_index_two = self.get_param('sheet_index_two') 
-        merge_key_column_id_two = self.get_param('merge_key_column_id_two') 
-        selected_column_ids_two = self.get_param('selected_column_ids_two')
+        how: str = self.get_param('how') 
+        sheet_index_one: int = self.get_param('sheet_index_one') 
+        sheet_index_two: int = self.get_param('sheet_index_two') 
+        merge_key_column_ids: List[List[ColumnID]] = self.get_param('merge_key_column_ids') 
+        selected_column_ids_one: List[ColumnID] = self.get_param('selected_column_ids_one') 
+        selected_column_ids_two: List[ColumnID] = self.get_param('selected_column_ids_two')
 
-        merge_key_one = self.prev_state.column_ids.get_column_header_by_id(sheet_index_one, merge_key_column_id_one)
-        merge_key_two = self.prev_state.column_ids.get_column_header_by_id(sheet_index_two, merge_key_column_id_two)
+        merge_keys_one: List[ColumnHeader] = self.prev_state.column_ids.get_column_headers_by_ids(sheet_index_one, list(map(lambda x: x[0], merge_key_column_ids)))
+        merge_keys_two: List[ColumnHeader] = self.prev_state.column_ids.get_column_headers_by_ids(sheet_index_two, list(map(lambda x: x[1], merge_key_column_ids)))
 
-        selected_columns_one = self.prev_state.column_ids.get_column_headers_by_ids(sheet_index_one, selected_column_ids_one)
-        selected_columns_two = self.prev_state.column_ids.get_column_headers_by_ids(sheet_index_two, selected_column_ids_two)
+        selected_column_headers_one: List[ColumnHeader] = self.prev_state.column_ids.get_column_headers_by_ids(sheet_index_one, selected_column_ids_one)
+        selected_column_headers_two: List[ColumnHeader] = self.prev_state.column_ids.get_column_headers_by_ids(sheet_index_two, selected_column_ids_two)
 
         # Update df indexes to start at 1
         df_one_name = self.post_state.df_names[sheet_index_one]
         df_two_name = self.post_state.df_names[sheet_index_two]
         df_new_name = self.post_state.df_names[len(self.post_state.dfs) - 1]
 
+        if len(merge_keys_one) == 0 and len(merge_keys_two) == 0:
+            return [f'{df_new_name} = pd.DataFrame()']
+
         # Now, we build the merge code 
         merge_code = []
         if how == 'lookup':
             # If the mege is a lookup, then we add the drop duplicates code
             temp_df_name = 'temp_df'
-            merge_code.append(f'{temp_df_name} = {df_two_name}.drop_duplicates(subset={column_header_to_transpiled_code(merge_key_two)}) # Remove duplicates so lookup merge only returns first match')
+            merge_code.append(f'{temp_df_name} = {df_two_name}.drop_duplicates(subset={column_header_list_to_transpiled_code(merge_keys_two)}) # Remove duplicates so lookup merge only returns first match')
             how_to_use = 'left'
         else:
             temp_df_name = df_two_name
             how_to_use = how
 
-
         # If we are only taking some columns, write the code to drop the ones we don't need!
-        deleted_columns_one = set(self.post_state.dfs[sheet_index_one].keys()).difference(set(selected_columns_one))
-        deleted_columns_two = set(self.post_state.dfs[sheet_index_two].keys()).difference(set(selected_columns_two))
+        deleted_columns_one = set(self.post_state.dfs[sheet_index_one].keys()).difference(set(selected_column_headers_one).union(set(merge_keys_one)))
+        deleted_columns_two = set(self.post_state.dfs[sheet_index_two].keys()).difference(set(selected_column_headers_two).union(set(merge_keys_two)))
+
         if len(deleted_columns_one) > 0:
             deleted_transpiled_column_header_one_list = column_header_list_to_transpiled_code(deleted_columns_one)
             merge_code.append(
@@ -86,15 +89,27 @@ class MergeCodeChunk(CodeChunk):
         # Finially append the merge
         if how == UNIQUE_IN_LEFT:
             merge_code.append(
-                f'{df_new_name} = {df_one_to_merge}.copy(deep=True)[~{df_one_to_merge}["{merge_key_one}"].isin({df_two_to_merge}["{merge_key_two}"])]'
+                f'{df_two_to_merge}_tmp = {df_two_to_merge}.drop_duplicates(subset={column_header_list_to_transpiled_code(merge_keys_two)})'
+            )
+            merge_code.append(
+                f'bool_index_array = {df_one_to_merge}.merge({df_two_to_merge}_tmp, left_on={column_header_list_to_transpiled_code(merge_keys_one)}, right_on={column_header_list_to_transpiled_code(merge_keys_two)}, how=\'left\', indicator=True)[\'_merge\'] == \'left_only\''
+            )
+            merge_code.append(
+                f'{df_new_name} = {df_one_to_merge}.copy(deep=True)[bool_index_array][{column_header_list_to_transpiled_code(selected_column_headers_one)}].reset_index(drop=True)'
             )
         elif how == UNIQUE_IN_RIGHT:
             merge_code.append(
-                f'{df_new_name} = {df_two_to_merge}.copy(deep=True)[~{df_two_to_merge}["{merge_key_two}"].isin({df_one_to_merge}["{merge_key_one}"])]'
+                f'{df_one_to_merge}_tmp = {df_one_to_merge}.drop_duplicates(subset={column_header_list_to_transpiled_code(merge_keys_one)})'
+            )
+            merge_code.append(
+                f'bool_index_array = {df_one_to_merge}_tmp.merge({df_two_to_merge}, left_on={column_header_list_to_transpiled_code(merge_keys_one)}, right_on={column_header_list_to_transpiled_code(merge_keys_two)}, how=\'right\', indicator=True)[\'_merge\'] == \'right_only\''
+            )
+            merge_code.append(
+                f'{df_new_name} = {df_two_to_merge}.copy(deep=True)[bool_index_array][{column_header_list_to_transpiled_code(selected_column_headers_two)}].reset_index(drop=True)'
             )
         else:      
             merge_code.append(
-                f'{df_new_name} = {df_one_to_merge}.merge({df_two_to_merge}, left_on=[{column_header_to_transpiled_code(merge_key_one)}], right_on=[{column_header_to_transpiled_code(merge_key_two)}], how=\'{how_to_use}\', suffixes=[\'_{suffix_one}\', \'_{suffix_two}\'])'
+                f'{df_new_name} = {df_one_to_merge}.merge({df_two_to_merge}, left_on={column_header_list_to_transpiled_code(merge_keys_one)}, right_on={column_header_list_to_transpiled_code(merge_keys_two)}, how=\'{how_to_use}\', suffixes=[\'_{suffix_one}\', \'_{suffix_two}\'])'
             )
 
         # And then return it
