@@ -39,8 +39,8 @@ from mitosheet.step_performers.delete_row import DeleteRowStepPerformer
 from mitosheet.step_performers.promote_row_to_header import PromoteRowToHeaderStepPerformer
 from mitosheet.step_performers.transpose import TransposeStepPerformer
 from mitosheet.step_performers.melt import MeltStepPerformer
-from mitosheet.step_performers.one_hot_encoding import OneHotEncodingStepPerformer
-from mitosheet.step_performers.set_dataframe_format import SetDataframeFormatStepPerformer
+from mitosheet.enterprise.step_performers.one_hot_encoding import OneHotEncodingStepPerformer
+from mitosheet.pro.step_performers.set_dataframe_format import SetDataframeFormatStepPerformer
 from mitosheet.step_performers.import_steps.dataframe_import import DataframeImportStepPerformer
 # AUTOGENERATED LINE: IMPORT (DO NOT DELETE)
 
