@@ -13,6 +13,7 @@ from typing import Any, Collection, Dict, List, Optional, Set, Tuple, Union
 import pandas as pd
 
 from mitosheet.data_in_mito import DataTypeInMito, get_data_type_in_mito
+from mitosheet.enterprise.mito_config import MitoConfig
 from mitosheet.experiments.experiment_utils import get_current_experiment
 from mitosheet.step_performers.import_steps.dataframe_import import DataframeImportStepPerformer
 from mitosheet.telemetry.telemetry_utils import log
@@ -165,7 +166,7 @@ class StepsManager:
     and parameters stay the same and are append-only.
     """
 
-    def __init__(self, args: Collection[Union[pd.DataFrame, str]], analysis_to_replay: Optional[str]=None):
+    def __init__(self, args: Collection[Union[pd.DataFrame, str]], mito_config: MitoConfig, analysis_to_replay: Optional[str]=None):
         """
         When initalizing the StepsManager, we also do preprocessing
         of the arguments that were passed to the mitosheet.
@@ -226,8 +227,20 @@ class StepsManager:
 
         # We also cache some of the sheet data in a form suitable to turn
         # into json, so that we can package it and send it to the front-end
-        # faster and with less work
-        self.saved_sheet_data: List[Dict] = []
+        # faster and with less work. Make sure to cache the starting values
+        # for the saved sheet data
+        self.saved_sheet_data: List[Dict] = dfs_to_array_for_json(
+            self.curr_step.final_defined_state,
+            set(range(len(args))),
+            [],
+            self.curr_step.dfs,
+            self.curr_step.df_names,
+            self.curr_step.df_sources,
+            self.curr_step.column_spreadsheet_code,
+            self.curr_step.column_filters,
+            self.curr_step.column_ids,
+            self.curr_step.df_formats,
+        )
         self.last_step_index_we_wrote_sheet_json_on = 0
 
         # We store the number of update events that have been processed successfully,
@@ -243,6 +256,9 @@ class StepsManager:
         # We store the experiment that is currently being run for this user
         self.experiment = get_current_experiment()
 
+        # We store the mito_config variables here so that we can use them in the api
+        self.mito_config = mito_config
+
     @property
     def curr_step(self) -> Step:
         """
@@ -465,7 +481,7 @@ class StepsManager:
 
         return last_valid_index
 
-    def execute_undo(self):
+    def execute_undo(self) -> None:
         """
         This function attempts to undo the most recent step, and if there
         is no most recent step, does nothing.
