@@ -16,10 +16,12 @@ from mitosheet.step_performers.graph_steps.plotly_express_graphs import DO_NOT_C
 from numpy import number
 
 import pandas as pd
-from mitosheet.mito_widget import MitoWidget, sheet
+from mitosheet.mito_backend import MitoBackend, get_mito_backend
 from mitosheet.parser import parse_formula
+from mitosheet.step_performers.pivot import PCT_NO_OP
 from mitosheet.transpiler.transpile import transpile
-from mitosheet.types import ColumnHeader, ColumnID, DataframeFormat, GraphID, MultiLevelColumnHeader, FilterOnColumnID, FilterOnColumnHeader
+from mitosheet.transpiler.transpile_utils import column_header_to_transpiled_code, column_header_list_to_transpiled_code
+from mitosheet.types import ColumnHeader, ColumnID, DataframeFormat, GraphID, MultiLevelColumnHeader, ColumnIDWithFilter, ColumnHeaderWithFilter, ColumnHeaderWithPivotTransform, ColumnIDWithPivotTransform
 from mitosheet.utils import NpEncoder, dfs_to_array_for_json, get_new_id
 
 
@@ -46,15 +48,15 @@ def check_dataframes_equal(test_wrapper):
     original_dfs = {
         df_name: df.copy(deep=True) for df, df_name in 
         zip(
-            test_wrapper.mito_widget.steps_manager.original_args,
-            test_wrapper.mito_widget.steps_manager.steps_including_skipped[0].df_names
+            test_wrapper.mito_backend.steps_manager.original_args,
+            test_wrapper.mito_backend.steps_manager.steps_including_skipped[0].df_names
         )
     }
     final_dfs = {
         df_name: df.copy(deep=True) for df, df_name in 
         zip(
-            test_wrapper.mito_widget.steps_manager.curr_step.dfs,
-            test_wrapper.mito_widget.steps_manager.curr_step.df_names
+            test_wrapper.mito_backend.steps_manager.curr_step.dfs,
+            test_wrapper.mito_backend.steps_manager.curr_step.df_names
         )
     }
 
@@ -67,35 +69,43 @@ def check_dataframes_equal(test_wrapper):
         test_wrapper.transpiled_code +
         [
             f'check_final_dataframe(\'{df_name}\', {df_name})'
-            for df_name in test_wrapper.mito_widget.steps_manager.curr_step.df_names
+            for df_name in test_wrapper.mito_backend.steps_manager.curr_step.df_names
         ]
     )
 
     import mitosheet
-    exec(code, 
-        {
-            'check_final_dataframe': check_final_dataframe,
-            # Make sure all the mitosheet functions are defined, which replaces the
-            # `from mitosheet import *` code that is at the top of all
-            # transpiled code 
-            **mitosheet.__dict__,
-        }, 
-        original_dfs
-    )
+    try:
+        exec(code, 
+            {
+                'check_final_dataframe': check_final_dataframe,
+                # Make sure all the mitosheet functions are defined, which replaces the
+                # `from mitosheet import *` code that is at the top of all
+                # transpiled code 
+                **mitosheet.__dict__,
+            }, 
+            original_dfs
+        )
+    except:
+        from mitosheet.errors import get_recent_traceback
+        print("Error executing code")
+        print(get_recent_traceback())
+        print("\nCode:")
+        print(code)
 
     # We then check that the sheet data json that is saved by the widget, which 
     # notably uses caching, does not get incorrectly cached and is written correctly
-    assert test_wrapper.mito_widget.sheet_data_json == json.dumps(dfs_to_array_for_json(
-        test_wrapper.mito_widget.steps_manager.curr_step.final_defined_state, 
-        set(i for i in range(len(test_wrapper.mito_widget.steps_manager.curr_step.dfs))),
+    sheet_data_json = test_wrapper.mito_backend.get_shared_state_variables()['sheet_data_json']
+    assert sheet_data_json == json.dumps(dfs_to_array_for_json(
+        test_wrapper.mito_backend.steps_manager.curr_step.final_defined_state, 
+        set(i for i in range(len(test_wrapper.mito_backend.steps_manager.curr_step.dfs))),
         [],
-        test_wrapper.mito_widget.steps_manager.curr_step.dfs,
-        test_wrapper.mito_widget.steps_manager.curr_step.df_names,
-        test_wrapper.mito_widget.steps_manager.curr_step.df_sources,
-        test_wrapper.mito_widget.steps_manager.curr_step.column_spreadsheet_code,
-        test_wrapper.mito_widget.steps_manager.curr_step.column_filters,
-        test_wrapper.mito_widget.steps_manager.curr_step.column_ids,
-        test_wrapper.mito_widget.steps_manager.curr_step.df_formats
+        test_wrapper.mito_backend.steps_manager.curr_step.dfs,
+        test_wrapper.mito_backend.steps_manager.curr_step.df_names,
+        test_wrapper.mito_backend.steps_manager.curr_step.df_sources,
+        test_wrapper.mito_backend.steps_manager.curr_step.column_spreadsheet_code,
+        test_wrapper.mito_backend.steps_manager.curr_step.column_filters,
+        test_wrapper.mito_backend.steps_manager.curr_step.column_ids,
+        test_wrapper.mito_backend.steps_manager.curr_step.df_formats
     ), cls=NpEncoder)
 
 
@@ -108,48 +118,52 @@ class MitoWidgetTestWrapper:
     set formulas, and get values to check the result.
     """
 
-    def __init__(self, mito_widget: MitoWidget):
-        self.mito_widget = mito_widget
+    def __init__(self, mito_backend: MitoBackend):
+        self.mito_backend = mito_backend
 
     @property
     def transpiled_code(self):
         # NOTE: we don't add comments to this testing functionality, so that 
         # we don't have to change tests if we update comments
-        return transpile(self.mito_widget.steps_manager, add_comments=False)
+        return transpile(self.mito_backend.steps_manager, add_comments=False)
     
     @property
     def optimized_code_chunks(self):
         # NOTE: we don't add comments to this testing functionality, so that 
         # we don't have to change tests if we update comments
-        return get_code_chunks(self.mito_widget.steps_manager.steps_including_skipped, optimize=True)
+        return get_code_chunks(self.mito_backend.steps_manager.steps_including_skipped, optimize=True)
 
     @property
     def curr_step_idx(self):
-        return self.mito_widget.steps_manager.curr_step_idx
+        return self.mito_backend.steps_manager.curr_step_idx
     
     @property
     def steps_including_skipped(self):
-        return self.mito_widget.steps_manager.steps_including_skipped
+        return self.mito_backend.steps_manager.steps_including_skipped
 
     @property
     def curr_step(self):
-        return self.mito_widget.steps_manager.curr_step
+        return self.mito_backend.steps_manager.curr_step
     
     @property
     def dfs(self):
-        return self.mito_widget.steps_manager.dfs
+        return self.mito_backend.steps_manager.dfs
 
     @property
     def df_names(self):
-        return self.mito_widget.steps_manager.curr_step.df_names
+        return self.mito_backend.steps_manager.curr_step.df_names
 
     @property
     def df_formats(self):
-        return self.mito_widget.steps_manager.curr_step.df_formats
+        return self.mito_backend.steps_manager.curr_step.df_formats
+    
+    @property
+    def sheet_data_json(self):
+        return self.mito_backend.steps_manager.sheet_data_json
 
     @property
-    def column_format_types(self):
-        return self.mito_widget.steps_manager.curr_step.column_format_types
+    def analysis_data_json(self):
+        return self.mito_backend.steps_manager.analysis_data_json
 
     @check_transpiled_code_after_call
     def add_column(self, sheet_index: int, column_header: str, column_header_index: int=-1) -> bool:
@@ -157,7 +171,7 @@ class MitoWidgetTestWrapper:
         Adds a column.
         """
 
-        return self.mito_widget.receive_message(self.mito_widget, {
+        return self.mito_backend.receive_message({
             'event': 'edit_event',
             'id': get_new_id(),
             'type': 'add_column_edit',
@@ -184,13 +198,12 @@ class MitoWidgetTestWrapper:
         if add_column:
             self.add_column(sheet_index, column_header)
 
-        column_id = self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(
+        column_id = self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(
             sheet_index,
             column_header
         )
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -216,21 +229,20 @@ class MitoWidgetTestWrapper:
         ) -> bool:
 
         merge_key_column_ids = list(map(lambda x: [
-            self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index_one, x[0]),
-            self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index_two, x[1]),
+            self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index_one, x[0]),
+            self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index_two, x[1]),
         ], merge_key_columns))
 
         selected_column_ids_one = [
-            self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index_one, column_header)
+            self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index_one, column_header)
             for column_header in selected_columns_one
         ]
         selected_column_ids_two = [
-            self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index_two, column_header)
+            self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index_two, column_header)
             for column_header in selected_columns_two
         ]
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -255,8 +267,7 @@ class MitoWidgetTestWrapper:
             sheet_indexes: int
         ) -> bool:
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -279,12 +290,11 @@ class MitoWidgetTestWrapper:
         ) -> bool:
 
         column_ids = [
-            self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index, column_header)
+            self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index, column_header)
             for column_header in column_headers
         ]
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -306,8 +316,7 @@ class MitoWidgetTestWrapper:
             labels: List[Union[int, str]],
         ) -> bool:        
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -329,8 +338,7 @@ class MitoWidgetTestWrapper:
         ) -> bool:
         
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -353,8 +361,7 @@ class MitoWidgetTestWrapper:
 
         
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -377,16 +384,15 @@ class MitoWidgetTestWrapper:
         ) -> bool:
 
         id_var_column_ids = [
-            self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index, column_header)
+            self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index, column_header)
             for column_header in id_var_column_headers
         ]
         value_var_column_ids = [
-            self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index, column_header)
+            self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index, column_header)
             for column_header in value_var_column_headers
         ]
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -409,14 +415,13 @@ class MitoWidgetTestWrapper:
             column_header: ColumnHeader,
         ) -> bool:
 
-        column_id =self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(
+        column_id =self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(
             sheet_index,
             column_header
         )
 
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -440,8 +445,7 @@ class MitoWidgetTestWrapper:
 
         
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -465,8 +469,7 @@ class MitoWidgetTestWrapper:
         ) -> bool:
 
         
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -489,10 +492,9 @@ class MitoWidgetTestWrapper:
             new_column_header_suffix: str
         ) -> bool:
 
-        column_id = self.mito_widget.steps_manager.curr_step.get_column_id_by_header(sheet_index, column_header)
+        column_id = self.mito_backend.steps_manager.curr_step.get_column_id_by_header(sheet_index, column_header)
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -516,12 +518,11 @@ class MitoWidgetTestWrapper:
         ) -> bool:
 
         column_ids = [
-            self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index, column_header)
+            self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(sheet_index, column_header)
             for column_header in column_headers
         ]
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -539,37 +540,56 @@ class MitoWidgetTestWrapper:
     def pivot_sheet(
             self, 
             sheet_index: int, 
-            pivot_rows: List[ColumnHeader],
-            pivot_columns: List[ColumnHeader],
+            # For convenience, you can use this testing API to either pass just column headers, or optionally
+            # column headers with transforms attached to them. This function turns them
+            # into the correct format before passing them to the backend
+            pivot_rows: Union[List[ColumnHeader], List[ColumnHeaderWithPivotTransform]],
+            pivot_columns: Union[List[ColumnHeader], List[ColumnHeaderWithPivotTransform]],
             values: Dict[ColumnHeader, List[str]],
-            pivot_filters: Optional[List[FilterOnColumnHeader]]=None,
+            pivot_filters: Optional[List[ColumnHeaderWithFilter]]=None,
             flatten_column_headers: bool=True,
             destination_sheet_index: Optional[int]=None,
             step_id: Optional[str]=None
         ) -> bool:
 
-        get_column_id_by_header = self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header
+        get_column_id_by_header = self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header
+
+        rows_ids_with_transforms: List[ColumnIDWithPivotTransform] = []
+        if len(pivot_rows) > 0 and isinstance(pivot_rows[0], str):
+            rows_ids_with_transforms = [{
+                'column_id': get_column_id_by_header(sheet_index, column_header), # type: ignore
+                'transformation': PCT_NO_OP
+            } for column_header in pivot_rows]
+        elif len(pivot_rows) > 0:
+            rows_ids_with_transforms = [{
+                'column_id': get_column_id_by_header(sheet_index, chwpt['column_header']), # type: ignore
+                'transformation': chwpt['transformation'] # type: ignore
+            } for chwpt in pivot_rows]
+        
+        column_ids_with_transforms: List[ColumnIDWithPivotTransform] = []
+        if len(pivot_columns) > 0 and isinstance(pivot_columns[0], str):
+            column_ids_with_transforms = [{
+                'column_id': get_column_id_by_header(sheet_index, column_header), # type: ignore
+                'transformation': PCT_NO_OP
+            } for column_header in pivot_columns]
+        elif len(pivot_columns) > 0:
+            column_ids_with_transforms = [{
+                'column_id': get_column_id_by_header(sheet_index, chwpt['column_header']), # type: ignore
+                'transformation': chwpt['transformation'] # type: ignore
+            } for chwpt in pivot_columns]
+
 
-        rows_ids = [
-            get_column_id_by_header(sheet_index, column_header)
-            for column_header in pivot_rows
-        ]
-        columns_ids = [
-            get_column_id_by_header(sheet_index, column_header)
-            for column_header in pivot_columns
-        ]
         values_column_ids_map = {
             get_column_id_by_header(sheet_index, column_header): value
             for column_header, value in values.items()
         }
 
-        pivot_filters_ids: List[FilterOnColumnID] = [
+        pivot_filters_ids: List[ColumnIDWithFilter] = [
             {'column_id': get_column_id_by_header(sheet_index, pf['column_header']), 'filter': pf['filter']} 
             for pf in pivot_filters
         ] if pivot_filters is not None else []
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -577,8 +597,8 @@ class MitoWidgetTestWrapper:
                 'step_id': get_new_id() if step_id is None else step_id,
                 'params': {
                     'sheet_index': sheet_index,
-                    'pivot_rows_column_ids': rows_ids,
-                    'pivot_columns_column_ids': columns_ids,
+                    'pivot_rows_column_ids_with_transforms': rows_ids_with_transforms,
+                    'pivot_columns_column_ids_with_transforms': column_ids_with_transforms,
                     'values_column_ids_map': values_column_ids_map,
                     'destination_sheet_index': destination_sheet_index,
                     'pivot_filters': pivot_filters_ids,
@@ -597,13 +617,12 @@ class MitoWidgetTestWrapper:
             value: Any
         ) -> bool:
 
-        column_id = self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(
+        column_id = self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(
             sheet_index,
             column_header
         )
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -631,13 +650,12 @@ class MitoWidgetTestWrapper:
             filters: List[Dict[str, Any]]
         ) -> bool:
 
-        column_id = self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(
+        column_id = self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(
             sheet_index,
             column_header
         )
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -661,13 +679,12 @@ class MitoWidgetTestWrapper:
             step_id: Optional[str]=None
         ) -> bool:
 
-        column_id = self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(
+        column_id = self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(
             sheet_index,
             column_header
         )
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -689,13 +706,12 @@ class MitoWidgetTestWrapper:
             new_column_index: int
         ) -> bool:
 
-        column_id = self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(
+        column_id = self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(
             sheet_index,
             column_header
         )
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -712,13 +728,12 @@ class MitoWidgetTestWrapper:
     @check_transpiled_code_after_call
     def rename_column(self, sheet_index: int, old_column_header: ColumnHeader, new_column_header: ColumnHeader, level: Optional[int]=None) -> bool:
 
-        column_id = self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(
+        column_id = self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(
             sheet_index,
             old_column_header
         )
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -735,13 +750,12 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def delete_columns(self, sheet_index: int, column_headers: List[ColumnHeader]) -> bool:
-        column_ids = [self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(
+        column_ids = [self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(
             sheet_index,
             column_header 
         ) for column_header in column_headers]
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -757,13 +771,12 @@ class MitoWidgetTestWrapper:
     @check_transpiled_code_after_call
     def change_column_dtype(self, sheet_index: int, column_headers: List[ColumnHeader], new_dtype: str) -> bool:
 
-        column_ids = self.mito_widget.steps_manager.curr_step.column_ids.get_column_ids_by_headers(
+        column_ids = self.mito_backend.steps_manager.curr_step.column_ids.get_column_ids_by_headers(
             sheet_index,
             column_headers
         )
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -787,8 +800,7 @@ class MitoWidgetTestWrapper:
         skiprows: Optional[List[int]]=None,
         error_bad_lines: Optional[List[bool]]=None
     ) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -807,8 +819,7 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def excel_import(self, file_name: str, sheet_names: List[str], has_headers: bool, skiprows: int, decimal: Optional[str]=None) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -826,8 +837,7 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def bulk_old_rename(self, move_to_deprecated_id_algorithm: bool=False) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -841,8 +851,7 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def undo(self) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'update_event',
                 'id': get_new_id(),
@@ -853,8 +862,7 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def redo(self) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'update_event',
                 'id': get_new_id(),
@@ -865,8 +873,7 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def clear(self) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'update_event',
                 'id': get_new_id(),
@@ -877,8 +884,7 @@ class MitoWidgetTestWrapper:
     
 
     def save_analysis(self, analysis_name: str) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'update_event',
                 'id': get_new_id(),
@@ -892,8 +898,7 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def delete_dataframe(self, sheet_index: int) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -907,8 +912,7 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def duplicate_dataframe(self, sheet_index: int) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -922,8 +926,7 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def rename_dataframe(self, sheet_index: int, new_dataframe_name: str) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -938,13 +941,12 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def set_cell_value(self, sheet_index: int, column_header: ColumnHeader, row_index: int, new_value: Any) -> bool:
-        column_id = self.mito_widget.steps_manager.curr_step.column_ids.get_column_id_by_header(
+        column_id = self.mito_backend.steps_manager.curr_step.column_ids.get_column_id_by_header(
             sheet_index,
             column_header
         )
 
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -961,8 +963,7 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def replay_analysis(self, analysis_name: str, step_import_data_list_to_overwrite: Optional[List[Dict[str, Any]]]=None) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'update_event',
                 'id': get_new_id(),
@@ -976,8 +977,7 @@ class MitoWidgetTestWrapper:
 
     @check_transpiled_code_after_call
     def checkout_step_by_idx(self, index: int) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'update_event',
                 'id': get_new_id(),
@@ -990,8 +990,7 @@ class MitoWidgetTestWrapper:
     
     @check_transpiled_code_after_call
     def checklist_update(self, checklist_id: str, completed_items: List[str], clear_other_items: bool) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'update_event',
                 'id': get_new_id(),
@@ -1141,8 +1140,7 @@ class MitoWidgetTestWrapper:
         if nbins is not None:
             params['graph_creation']['nbins'] = nbins
 
-        return self.mito_widget.receive_message(
-            self.mito_widget, 
+        return self.mito_backend.receive_message( 
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -1153,8 +1151,7 @@ class MitoWidgetTestWrapper:
         )
 
     def delete_graph(self, graph_id: GraphID) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -1167,8 +1164,7 @@ class MitoWidgetTestWrapper:
         )
 
     def duplicate_graph(self, old_graph_id: GraphID, new_graph_id: GraphID) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -1183,8 +1179,7 @@ class MitoWidgetTestWrapper:
 
 
     def rename_graph(self, graph_id: GraphID, new_graph_tab_name: str) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'edit_event',
                 'id': get_new_id(),
@@ -1198,8 +1193,7 @@ class MitoWidgetTestWrapper:
         )
 
     def update_existing_imports(self, updated_import_objs: List[Dict[str, Any]]) -> bool:
-        return self.mito_widget.receive_message(
-            self.mito_widget,
+        return self.mito_backend.receive_message(
             {
                 'event': 'update_event',
                 'type': 'update_existing_import_update',
@@ -1215,22 +1209,22 @@ class MitoWidgetTestWrapper:
         Gets the formula for a given column. Returns an empty
         string if nothing exists.
         """
-        column_id = self.mito_widget.steps_manager.curr_step.get_column_id_by_header(
+        column_id = self.mito_backend.steps_manager.curr_step.get_column_id_by_header(
             sheet_index, column_header
         )
-        if column_id not in self.mito_widget.steps_manager.curr_step.column_spreadsheet_code[sheet_index]:
+        if column_id not in self.mito_backend.steps_manager.curr_step.column_spreadsheet_code[sheet_index]:
             return ''
-        return self.mito_widget.steps_manager.curr_step.column_spreadsheet_code[sheet_index][column_id]
+        return self.mito_backend.steps_manager.curr_step.column_spreadsheet_code[sheet_index][column_id]
 
     def get_python_formula(self, sheet_index: int, column_header: ColumnHeader) -> str:
         """
         Gets the formula for a given column. Returns an empty
         string if nothing exists.
         """
-        column_id = self.mito_widget.steps_manager.curr_step.get_column_id_by_header(
+        column_id = self.mito_backend.steps_manager.curr_step.get_column_id_by_header(
             sheet_index, column_header
         )
-        if column_id not in self.mito_widget.steps_manager.curr_step.column_spreadsheet_code[sheet_index]:
+        if column_id not in self.mito_backend.steps_manager.curr_step.column_spreadsheet_code[sheet_index]:
             return ''
 
         column_headers = self.curr_step.post_state.dfs[sheet_index].keys()
@@ -1251,7 +1245,7 @@ class MitoWidgetTestWrapper:
 
         Errors if the value does not exist
         """
-        return self.mito_widget.steps_manager.curr_step.dfs[sheet_index].at[row - 1, column_header]
+        return self.mito_backend.steps_manager.curr_step.dfs[sheet_index].at[row - 1, column_header]
 
     def get_column(self, sheet_index: int, column_header: ColumnHeader, as_list: bool) -> Union[pd.Series, List]:
         """
@@ -1261,15 +1255,15 @@ class MitoWidgetTestWrapper:
         Errors if the column does not exist. 
         """
         if as_list:
-            return self.mito_widget.steps_manager.dfs[sheet_index][column_header].tolist()
-        return self.mito_widget.steps_manager.dfs[sheet_index][column_header]
+            return self.mito_backend.steps_manager.dfs[sheet_index][column_header].tolist()
+        return self.mito_backend.steps_manager.dfs[sheet_index][column_header]
 
     def get_graph_data(self, graph_id: str) -> Dict[str, Dict[str, Any]]: 
         """
         Returns the graph_data object 
         """
-        if graph_id in self.mito_widget.steps_manager.curr_step.final_defined_state.graph_data_dict.keys():
-            return self.mito_widget.steps_manager.curr_step.final_defined_state.graph_data_dict[graph_id]
+        if graph_id in self.mito_backend.steps_manager.curr_step.final_defined_state.graph_data_dict.keys():
+            return self.mito_backend.steps_manager.curr_step.final_defined_state.graph_data_dict[graph_id]
         else:
             return {}
 
@@ -1375,7 +1369,7 @@ class MitoWidgetTestWrapper:
         """
         Returns the DataframeFormat object for a specific sheet
         """
-        return self.mito_widget.steps_manager.curr_step.final_defined_state.df_formats[sheet_index]
+        return self.mito_backend.steps_manager.curr_step.final_defined_state.df_formats[sheet_index]
         
 
 def create_mito_wrapper(sheet_one_A_data: List[Any], sheet_two_A_data: Optional[List[Any]]=None) -> MitoWidgetTestWrapper:
@@ -1391,16 +1385,16 @@ def create_mito_wrapper(sheet_one_A_data: List[Any], sheet_two_A_data: Optional[
     if sheet_two_A_data is not None:
         dfs.append(pd.DataFrame(data={'A': sheet_two_A_data}))
 
-    mito_widget = sheet(*dfs)
-    return MitoWidgetTestWrapper(mito_widget)
+    mito_backend = get_mito_backend(*dfs)
+    return MitoWidgetTestWrapper(mito_backend)
 
 def create_mito_wrapper_dfs(*args: pd.DataFrame) -> MitoWidgetTestWrapper:
     """
     Creates a MitoWidgetTestWrapper with a mito instance with the given
     data frames.
     """
-    mito_widget = sheet(*args)
-    return MitoWidgetTestWrapper(mito_widget)
+    mito_backend = get_mito_backend(*args)
+    return MitoWidgetTestWrapper(mito_backend)
 
 def make_multi_index_header_df(data: Dict[Union[str, int], List[Any]], column_headers: List[ColumnHeader], index: Optional[List[Any]]=None) -> pd.DataFrame:
     """
@@ -1433,4 +1427,23 @@ def make_multi_index_header_df(data: Dict[Union[str, int], List[Any]], column_he
     df.columns = pd.MultiIndex.from_tuples(final_column_headers)
     if index is not None:
         df.index = index
-    return df
\ No newline at end of file
+    return df
+
+def get_dataframe_generation_code(df: pd.DataFrame) -> str:
+    """
+    Given a dataframe like:
+
+        date (year)      value sum
+    0         2000             1.0
+    1         2001             NaN
+
+    Will return the string:
+    pd.DataFrame({'date (year)': [2000, 2001], 'value sum: [1.0, np.NaN]})
+
+    This is useful when you have a dataframe you want to create at runtime and then put into a test.
+    """
+    OPEN_BRACKET = "{"
+    CLOSE_BRACKET = "}"
+
+    data = ", ".join([f"{column_header_to_transpiled_code(column_header)}: {column_header_list_to_transpiled_code(df[column_header].to_list())}" for column_header in df.columns])
+    return f'pd.DataFrame({OPEN_BRACKET}{data}{CLOSE_BRACKET})'
